/*
Sketch Kesselhaus OHS, Stand 09/2014
 Version 1.01
 Umfang:
 6* DS18B20 für die Heizungsvorläufe  über 1-wire        Digital PIN: 5
 2* DHTnn für Aussen und Innentemperatur                 Digital PIN: 24,26
 1* s0 Stromverbrauch Hauptkompressor                    Digital PIN: 40
 1* s0 Schaltzeiten Hauptkompressor                      Digital PIN: 30
 1* s0 Nachspeisung Heizsystem Wasseruhr mit Impulsgeber Digital PIN: 31
 Ethernet-Shield DHCP und WEB-Server
 */

// Initialisierung
// Dallas DS18B20
#include <OneWire.h>
#include <DallasTemperature.h>
#define ONE_WIRE_BUS 5                 //Digitaler Anschluss an PIN 5
OneWire oneWire(ONE_WIRE_BUS);         // Initialisierung einer oneWire Instanz für die Devices
DallasTemperature sensoren(&oneWire);  // Anbindung oneWire an Dallas DS18B20
// Sensoren deklarieren
DeviceAddress Sensor_1 = { 
  0x28, 0x59, 0x69, 0x82, 0x05, 0x00, 0x00, 0xAF };
DeviceAddress Sensor_2 = { 
  0x28, 0x07, 0xCA, 0x81, 0x05, 0x00, 0x00, 0x67};
DeviceAddress Sensor_3 = { 
  0x28, 0x37, 0x5C, 0x82, 0x05, 0x00, 0x00, 0x29};
DeviceAddress Sensor_4 = { 
  0x28, 0x4C, 0x52, 0x82, 0x05, 0x00, 0x00, 0xEB};
DeviceAddress Sensor_5 = { 
  0x28, 0xB1, 0xE4, 0x81, 0x05, 0x00, 0x00, 0x88};
DeviceAddress Sensor_6 = { 
  0x28, 0x9E, 0x6B, 0x82, 0x05, 0x00, 0x00, 0x36};

String SensorBenennung[7] = {
  "HK_Produktion","HK_Lackierung","HK_HALLE_Sozialbereich","HK_Verwaltung","Ruecklauf","Vorlauf","Arduino"};
// DHTnn
// die Biblithek wurde um die Taupunkt und Wassermengenberechnug erweiter
#include "DHT.h"                      // DHT & AM2302 Bibliothek
DHT dht_1;
DHT dht_2;
String DHTBenennung[2] = {
  "KH_innen","KW_aussen"};
// Array für die Speicherung der Temperaturdaten
float temperaturen[10][4]; //Temperatur;Luftfeuchte;Taupunkt;Wassermende
int PageRequest = 1; //Nur bei der 5 Anfrage
// s0 Messungen
const int pinCount = 4;
const byte s0Pins[pinCount] = { 
  30, 31, 40, 41};
String s0PinsBenennung[pinCount] = { 
  "Gasimpuls", "KompressorImpuls", "KompressorSchaltung", "WU_Nachspeisung"};
byte lastState[pinCount];  
unsigned long counterValue[pinCount];

// Netzwerk
#include <Ethernet.h>
#include <SPI.h>
byte mac[] = { 
  0xDA, 0xAD, 0xBE, 0xEF, 0xFB, 0xEA };
EthernetClient  client;
EthernetServer server(80);
char c;                               // gesendetes zeichen des Client
String readString;                    // durch den Client angeforderte URL
byte ip[] = { 
  192, 168, 1, 177 };
int DHCP_Request = 0;
void setup() {
  sensoren.begin();                   // Dallas Biblithek starten
  dht_1.setup(26);                    // data pin 24
  dht_2.setup(24);                    // data pin 25
  delay(1000);
  TemperaturMessung();                // Temperaturarray initialisieren
  Serial.begin(9600);                 // Seriellen Port starten
  for (byte i = 0; i < pinCount; i++) {
    pinMode(s0Pins[i], INPUT_PULLUP);
    digitalWrite(s0Pins[i], LOW);
    lastState[i] = 0;
  }

  // Netzwerk initialisieren und starten
  Serial.println("Konfiguriere Ethernet via DHCP");
  if (Ethernet.begin(mac) == 0) {
    while (1) {
      Serial.println("Fehler: keine DHCP-Antwort");
      DHCP_Request++;
      delay(2000);                    //warten und erneut versuchen
      if (DHCP_Request > 3) {
        Serial.println("Failed to configure Ethernet using DHCP");
        Serial.print("IP number assigned is ");
        Ethernet.begin(mac, ip);
        break;
      }
    }
  }
  Serial.print("IP Adresse ist:  ");
  Serial.println(Ethernet.localIP());
  server.begin();
}

void loop() {
  for (byte i = 0; i < pinCount  ; i++) {
    byte val = digitalRead(s0Pins[i]);
    if (val == HIGH and lastState[i] == LOW) {
      Serial.println("Impuls erhalten");
      counterValue[i]++;
      Serial.println(counterValue[i]);
    }
    lastState[i] = digitalRead(s0Pins[i]);
  } 

  // Anfragen an den Webserver
  client = server.available();
  if (client) {     
    WebServerRequest();   
    delay(1);
    client.flush();
    delay(1);
    client.stop();
  }
}

void TemperaturMessung() // temperatursensoren auslesen und im einem Array ablegen
{
  // DS18B20
  sensoren.requestTemperatures(); // Temperaturen abrufen
  temperaturen[1][0] = sensoren.getTempC(Sensor_1); 
  Serial.println(sensoren.getTempC(Sensor_1)); 
  temperaturen[2][0] = sensoren.getTempC(Sensor_2); 
  temperaturen[3][0] = sensoren.getTempC(Sensor_3); 
  temperaturen[4][0] = sensoren.getTempC(Sensor_4); 
  temperaturen[5][0] = sensoren.getTempC(Sensor_5); 
  temperaturen[6][0] = sensoren.getTempC(Sensor_6); 
  // DHTnn
  temperaturen[7][0] = dht_1.getTemperature();
  temperaturen[7][1] = dht_1.getHumidity();
  temperaturen[7][2] = dht_1.getDewPoint();
  temperaturen[7][3] = dht_1.getWaterVolume();

  temperaturen[8][0] = dht_2.getTemperature();
  temperaturen[8][1] = dht_2.getHumidity();
  temperaturen[8][2] = dht_2.getDewPoint();
  temperaturen[8][3] = dht_2.getWaterVolume();
 
  PageRequest = 1 ;


}

void WebServerRequest() {
  Serial.println("new client");
  // an http request ends with a blank line
  boolean currentLineIsBlank = true;
  while (client.connected()) {
    if (client.available()) {
      char c = client.read();
      if (readString.length() < 600) {
        readString += c;
      }
      if (c == '\n' && currentLineIsBlank) {
        Serial.println(readString);
        // send a standard http response header
        if (readString.indexOf("/?pull") > 0) //checks for
        {
          sendPage();
          ;
          break;
        }
        if (readString.indexOf("/") > 0) //checks for
        {
          sendPageNO();
          break;
        }
        break;
      }
      if (c == '\n') {
        // you're starting a new line
        currentLineIsBlank = true;
      }
      else if (c != '\r') {        
        currentLineIsBlank = false; // you've gotten a character on the current line
      }
    }
  }
  readString = "";
} 


void sendPage() {
  // send a standard http response header
  client.println("HTTP/1.1 200 OK");
  client.println("<head><meta Content-Type: text/html>");
  client.println("Connection: close");  // the connection will be closed after completion of the response
  client.println();
  client.println("<!DOCTYPE HTML>");
  client.println("</head><body><html>");
  TemperaturMessung();
  printPage();
  counterValue[0]=0;
  counterValue[1]=0; 
  counterValue[3]=0; 
  client.println("\r\n\r\n ");
  client.println(" ");
  client.println("<p>Data reset");
  client.println("</body></html>");
}


void sendPageNO() {
  // send a standard http response header
  client.println("HTTP/1.1 200 OK");
  client.println("<head><meta Content-Type: text/html>");
  client.println("Connection: close");  // the connection will be closed after completion of the response
  client.println();
  client.println("<!DOCTYPE HTML>");
  client.println("<META HTTP-EQUIV='refresh' CONTENT='300'>");
  client.println("</head><body><html>");
  client.println("\r\n\r\n ");
  client.println("<p>");
  printPage();
  client.println("\r\n\r\n ");
  client.println(" ");
  client.println("<p>No Data reset");
  client.println("</body></html>");
}



void printPage(){
  client.print("DS18B20: ");
  client.print(temperaturen[1][0]);
  client.print(" ");
  client.print(temperaturen[2][0]);
  client.print(" ");
  client.print(temperaturen[3][0]);
  client.print(" ");
  client.print(temperaturen[4][0]);
  client.print(" ");
  client.print(temperaturen[5][0]);
  client.print(" ");
  client.print(temperaturen[6][0]);
  client.print(" ");
  client.println("\r\n\r\n ");
  client.println("<p>");

  client.print("DHT1: "); 
  client.print(temperaturen[7][0]);
  client.print(" ");
  client.print(temperaturen[7][1]);
  client.print(" ");
  client.print(temperaturen[7][2]);
  client.print(" ");
  client.print(temperaturen[7][3]);
  client.print(" ");
  client.println("\r\n\r\n <p> ");

  client.print("DHT2: "); 
  client.print(temperaturen[8][0]);
  client.print(" ");
  client.print(temperaturen[8][1]);
  client.print(" ");
  client.print(temperaturen[8][2]);
  client.print(" ");
  client.print(temperaturen[8][3]);
  client.print(" ");
  client.println("\r\n\r\n <p> ");   

  client.print("s0: "); 
  client.print(counterValue[0]);
  client.print(" ");
  client.print(counterValue[1]);
  client.print(" ");
  client.print(counterValue[2]);
  client.print(" ");
  client.print(counterValue[3]);
  client.print(" ");
  client.println("\r\n\r\n <p> ");      
}







